@@ -1,24 +1,20 @@
 <?php
-//
-// +----------------------------------------------------------------------+
-// | PHP Version 5                                                        |
-// +----------------------------------------------------------------------+
-// | Copyright (c) 1997-2004 The PHP Group                                |
-// +----------------------------------------------------------------------+
-// | This source file is subject to version 3.0 of the PHP license,       |
-// | that is bundled with this package in the file LICENSE, and is        |
-// | available through the world-wide-web at the following url:           |
-// | http://www.php.net/license/3_0.txt.                                  |
-// | If you did not receive a copy of the PHP license and are unable to   |
-// | obtain it through the world-wide-web, please send a note to          |
-// | license@php.net so we can mail you a copy immediately.               |
-// +----------------------------------------------------------------------+
-// | Authors: Stig Bakken <ssb@php.net>                                   |
-// |          Tomas V.V.Cox <cox@idecnet.com>                             |
-// |                                                                      |
-// +----------------------------------------------------------------------+
-//
-// $Id: pearcmd.php,v 1.39 2008/03/25 04:58:42 cellog Exp $
+/**
+ * PEAR, the PHP Extension and Application Repository
+ *
+ * Command line interface
+ *
+ * PHP versions 4 and 5
+ *
+ * @category   pear
+ * @package    PEAR
+ * @author     Stig Bakken <ssb@php.net>
+ * @author     Tomas V.V.Cox <cox@idecnet.com>
+ * @copyright  1997-2009 The Authors
+ * @license    http://opensource.org/licenses/bsd-license.php New BSD License
+ * @version    CVS: $Id: pearcmd.php 313023 2011-07-06 19:17:11Z dufuz $
+ * @link       http://pear.php.net/package/PEAR
+ */
 
 ob_end_clean();
 if (!defined('PEAR_RUNTYPE')) {
@@ -47,7 +43,7 @@ ob_implicit_flush(true);
 $_PEAR_PHPDIR = '#$%^&*';
 set_error_handler('error_handler');
 
-$pear_package_version = "1.7.2";
+$pear_package_version = "1.9.4";
 
 require_once 'PEAR.php';
 require_once 'PEAR/Frontend.php';
@@ -64,6 +60,7 @@ if (!isset($_SERVER['argv']) && !isset($argv) && !isset($HTTP_SERVER_VARS['argv'
     echo 'ERROR: either use the CLI php executable, or set register_argc_argv=On in php.ini';
     exit(1);
 }
+
 $argv = Console_Getopt::readPHPArgv();
 // fix CGI sapi oddity - the -- in pear.bat/pear is not removed
 if (php_sapi_name() != 'cli' && isset($argv[1]) && $argv[1] == '--') {
@@ -71,12 +68,8 @@ if (php_sapi_name() != 'cli' && isset($argv[1]) && $argv[1] == '--') {
     $argv = array_values($argv);
 }
 $progname = PEAR_RUNTYPE;
-if (in_array('getopt2', get_class_methods('Console_Getopt'))) {
-    array_shift($argv);
-    $options = Console_Getopt::getopt2($argv, "c:C:d:D:Gh?sSqu:vV");
-} else {
-    $options = Console_Getopt::getopt($argv, "c:C:d:D:Gh?sSqu:vV");
-}
+array_shift($argv);
+$options = Console_Getopt::getopt2($argv, "c:C:d:D:Gh?sSqu:vV");
 if (PEAR::isError($options)) {
     usage($options);
 }
@@ -249,7 +242,6 @@ if ($store_user_config) {
 }
 
 $command = (isset($options[1][0])) ? $options[1][0] : null;
-
 if (empty($command) && ($store_user_config || $store_system_config)) {
     exit;
 }
@@ -265,6 +257,7 @@ if ($fetype == 'Gtk' || $fetype == 'Gtk2') {
     if ($command == 'help') {
         usage(null, @$options[1][1]);
     }
+
     if (!$config->validConfiguration()) {
         PEAR::raiseError('CRITICAL ERROR: no existing valid configuration files found in files ' .
             "'$pear_user_config' or '$pear_system_config', please copy an existing configuration" .
@@ -280,15 +273,13 @@ if ($fetype == 'Gtk' || $fetype == 'Gtk2') {
 
     $short_args = $long_args = null;
     PEAR_Command::getGetoptArgs($command, $short_args, $long_args);
-    if (in_array('getopt2', get_class_methods('Console_Getopt'))) {
-        array_shift($options[1]);
-        $tmp = Console_Getopt::getopt2($options[1], $short_args, $long_args);
-    } else {
-        $tmp = Console_Getopt::getopt($options[1], $short_args, $long_args);
-    }
+    array_shift($options[1]);
+    $tmp = Console_Getopt::getopt2($options[1], $short_args, $long_args);
+
     if (PEAR::isError($tmp)) {
         break;
     }
+
     list($tmpopt, $params) = $tmp;
     $opts = array();
     foreach ($tmpopt as $foo => $tmp2) {
@@ -296,10 +287,11 @@ if ($fetype == 'Gtk' || $fetype == 'Gtk2') {
         if ($value === null) {
             $value = true; // options without args
         }
+
         if (strlen($opt) == 1) {
             $cmdoptions = $cmd->getOptions($command);
             foreach ($cmdoptions as $o => $d) {
-                if (@$d['shortopt'] == $opt) {
+                if (isset($d['shortopt']) && $d['shortopt'] == $opt) {
                     $opts[$o] = $value;
                 }
             }
@@ -309,10 +301,12 @@ if ($fetype == 'Gtk' || $fetype == 'Gtk2') {
             }
         }
     }
+
     $ok = $cmd->run($command, $opts, $params);
     if ($ok === false) {
         PEAR::raiseError("unknown command `$command'");
     }
+
     if (PEAR::isError($ok)) {
         PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, array($ui, "displayFatalError"));
         PEAR::raiseError($ok);
@@ -324,17 +318,17 @@ if ($fetype == 'Gtk' || $fetype == 'Gtk2') {
 function usage($error = null, $helpsubject = null)
 {
     global $progname, $all_commands;
-    $stderr = fopen('php://stderr', 'w');
+    $stdout = fopen('php://stdout', 'w');
     if (PEAR::isError($error)) {
-        fputs($stderr, $error->getMessage() . "\n");
+        fputs($stdout, $error->getMessage() . "\n");
     } elseif ($error !== null) {
-        fputs($stderr, "$error\n");
+        fputs($stdout, "$error\n");
     }
+
     if ($helpsubject != null) {
         $put = cmdHelp($helpsubject);
     } else {
-        $put =
-            "Commands:\n";
+        $put = "Commands:\n";
         $maxlen = max(array_map("strlen", $all_commands));
         $formatstr = "%-{$maxlen}s  %s\n";
         ksort($all_commands);
@@ -347,8 +341,12 @@ function usage($error = null, $helpsubject = null)
             "Type \"$progname help shortcuts\" to list all command shortcuts.\n".
             "Type \"$progname help <command>\" to get the help for the specified command.";
     }
-    fputs($stderr, "$put\n");
-    fclose($stderr);
+    fputs($stdout, "$put\n");
+    fclose($stdout);
+
+    if ($error === null) {
+        exit(0);
+    }
     exit(1);
 }
 
@@ -388,12 +386,14 @@ function cmdHelp($command)
         if (is_string($help)) {
             return "$progname $command [options] $help\n";
         }
+
         if ($help[1] === null) {
             return "$progname $command $help[0]";
-        } else {
-            return "$progname $command [options] $help[0]\n$help[1]";
         }
+
+        return "$progname $command [options] $help[0]\n$help[1]";
     }
+
     return "Command '$command' is not valid, try '$progname help'";
 }
 
@@ -445,6 +445,4 @@ function error_handler($errno, $errmsg, $file, $line, $vars) {
  * mode: php
  * End:
  */
-// vim600:syn=php
-
-?>
+// vim600:syn=php
\ No newline at end of file
